package scripts;

import com.stencyl.graphics.G;
import com.stencyl.graphics.BitmapWrapper;

import com.stencyl.behavior.Script;
import com.stencyl.behavior.Script.*;
import com.stencyl.behavior.ActorScript;
import com.stencyl.behavior.SceneScript;
import com.stencyl.behavior.TimedTask;

import com.stencyl.models.Actor;
import com.stencyl.models.GameModel;
import com.stencyl.models.actor.Animation;
import com.stencyl.models.actor.ActorType;
import com.stencyl.models.actor.Collision;
import com.stencyl.models.actor.Group;
import com.stencyl.models.Scene;
import com.stencyl.models.Sound;
import com.stencyl.models.Region;
import com.stencyl.models.Font;
import com.stencyl.models.Joystick;

import com.stencyl.Engine;
import com.stencyl.Input;
import com.stencyl.Key;
import com.stencyl.utils.Utils;

import openfl.ui.Mouse;
import openfl.display.Graphics;
import openfl.display.BlendMode;
import openfl.display.BitmapData;
import openfl.display.Bitmap;
import openfl.events.Event;
import openfl.events.KeyboardEvent;
import openfl.events.TouchEvent;
import openfl.net.URLLoader;

import box2D.common.math.B2Vec2;
import box2D.dynamics.B2Body;
import box2D.dynamics.B2Fixture;
import box2D.dynamics.joints.B2Joint;
import box2D.collision.shapes.B2Shape;

import motion.Actuate;
import motion.easing.Back;
import motion.easing.Cubic;
import motion.easing.Elastic;
import motion.easing.Expo;
import motion.easing.Linear;
import motion.easing.Quad;
import motion.easing.Quart;
import motion.easing.Quint;
import motion.easing.Sine;

import com.stencyl.graphics.shaders.BasicShader;
import com.stencyl.graphics.shaders.GrayscaleShader;
import com.stencyl.graphics.shaders.SepiaShader;
import com.stencyl.graphics.shaders.InvertShader;
import com.stencyl.graphics.shaders.GrainShader;
import com.stencyl.graphics.shaders.ExternalShader;
import com.stencyl.graphics.shaders.InlineShader;
import com.stencyl.graphics.shaders.BlurShader;
import com.stencyl.graphics.shaders.SharpenShader;
import com.stencyl.graphics.shaders.ScanlineShader;
import com.stencyl.graphics.shaders.CSBShader;
import com.stencyl.graphics.shaders.HueShader;
import com.stencyl.graphics.shaders.TintShader;
import com.stencyl.graphics.shaders.BloomShader;



class Design_119_119_SceneGamepadSetup extends SceneScript
{
	public var _AnalogPressureH:Float;
	public var _AnalogPressureV:Float;
	public var _AnalogDegree:Float;
	public var _AnalogDistance:Float;
	
	
	public function new(dummy:Int, dummy2:Engine)
	{
		super();
		nameMap.set("AnalogPressureH", "_AnalogPressureH");
		_AnalogPressureH = 0.0;
		nameMap.set("AnalogPressureV", "_AnalogPressureV");
		_AnalogPressureV = 0.0;
		nameMap.set("AnalogDegree", "_AnalogDegree");
		_AnalogDegree = 0.0;
		nameMap.set("AnalogDistance", "_AnalogDistance");
		_AnalogDistance = 0.0;
		
	}
	
	override public function init()
	{
		
		/* ======================== When Creating ========================= */
		/* "" */ Input.enableJoystick();
		/* "Sets deadzone for stick, or keep it to 0 and manually check " */ Input.setJoySensitivity(5 / 100);
		Input.mapJoystickButton("0, left hat", "left");
		Input.mapJoystickButton("0, right hat", "right");
		Input.mapJoystickButton("0, up hat", "up");
		Input.mapJoystickButton("0, down hat", "down");
		Input.mapJoystickButton("0, -axis 0", "Left_Analog");
		Input.mapJoystickButton("0, +axis 0", "Right_Analog");
		Input.mapJoystickButton("0, -axis 1", "Up_Analog");
		Input.mapJoystickButton("0, +axis 1", "Down_Analog");
		Input.mapJoystickButton("0, 0", "Attack");
		
		/* ======================== When Updating ========================= */
		addWhenUpdatedListener(null, function(elapsedTime:Float, list:Array<Dynamic>):Void
		{
			if(wrapper.enabled)
			{
				/* Analog stick control detection in here. */
				if(isKeyDown("Right_Analog"))
				{
					_AnalogPressureH = Input.getButtonPressure("Right_Analog");
				}
				else if(isKeyDown("Left_Analog"))
				{
					_AnalogPressureH = -(Input.getButtonPressure("Left_Analog"));
				}
				else
				{
					_AnalogPressureH = 0;
				}
				if(isKeyDown("Up_Analog"))
				{
					_AnalogPressureV = -(Input.getButtonPressure("Up_Analog"));
				}
				else if(isKeyDown("Down_Analog"))
				{
					_AnalogPressureV = Input.getButtonPressure("Down_Analog");
				}
				else
				{
					_AnalogPressureV = 0;
				}
				/* "Sets direction degree." */ _AnalogDegree = Math.round(Utils.DEG * (Math.atan2(_AnalogPressureV, _AnalogPressureH)));
				/* Distance from center analog range: 0 -> 1 */
				_AnalogDistance = asNumber(Math.sqrt((Math.pow(_AnalogPressureH, 2) + Math.pow(_AnalogPressureV, 2))));
				/* "Transforms the top-negative degrees into positive, so it's a 'full' 360 degrees." */ if((_AnalogDegree < 0))
				{
					_AnalogDegree = (360 - Math.abs(_AnalogDegree));
				}
			}
		});
		
	}
	
	override public function forwardMessage(msg:String)
	{
		
	}
}