package scripts;

import com.stencyl.graphics.G;
import com.stencyl.graphics.BitmapWrapper;

import com.stencyl.behavior.Script;
import com.stencyl.behavior.Script.*;
import com.stencyl.behavior.ActorScript;
import com.stencyl.behavior.SceneScript;
import com.stencyl.behavior.TimedTask;

import com.stencyl.models.Actor;
import com.stencyl.models.GameModel;
import com.stencyl.models.actor.Animation;
import com.stencyl.models.actor.ActorType;
import com.stencyl.models.actor.Collision;
import com.stencyl.models.actor.Group;
import com.stencyl.models.Scene;
import com.stencyl.models.Sound;
import com.stencyl.models.Region;
import com.stencyl.models.Font;
import com.stencyl.models.Joystick;

import com.stencyl.Engine;
import com.stencyl.Input;
import com.stencyl.Key;
import com.stencyl.utils.Utils;

import openfl.ui.Mouse;
import openfl.display.Graphics;
import openfl.display.BlendMode;
import openfl.display.BitmapData;
import openfl.display.Bitmap;
import openfl.events.Event;
import openfl.events.KeyboardEvent;
import openfl.events.TouchEvent;
import openfl.net.URLLoader;

import box2D.common.math.B2Vec2;
import box2D.dynamics.B2Body;
import box2D.dynamics.B2Fixture;
import box2D.dynamics.joints.B2Joint;

import motion.Actuate;
import motion.easing.Back;
import motion.easing.Cubic;
import motion.easing.Elastic;
import motion.easing.Expo;
import motion.easing.Linear;
import motion.easing.Quad;
import motion.easing.Quart;
import motion.easing.Quint;
import motion.easing.Sine;

import com.stencyl.graphics.shaders.BasicShader;
import com.stencyl.graphics.shaders.GrayscaleShader;
import com.stencyl.graphics.shaders.SepiaShader;
import com.stencyl.graphics.shaders.InvertShader;
import com.stencyl.graphics.shaders.GrainShader;
import com.stencyl.graphics.shaders.ExternalShader;
import com.stencyl.graphics.shaders.InlineShader;
import com.stencyl.graphics.shaders.BlurShader;
import com.stencyl.graphics.shaders.SharpenShader;
import com.stencyl.graphics.shaders.ScanlineShader;
import com.stencyl.graphics.shaders.CSBShader;
import com.stencyl.graphics.shaders.HueShader;
import com.stencyl.graphics.shaders.TintShader;
import com.stencyl.graphics.shaders.BloomShader;



class Design_24_24_CameraRelative extends ActorScript
{
	public var _OriginalSpeed:Float;
	public var _Speed:Float;
	public var _ChangeNumbCheck:Float;
	public var _FastSpeedChanging:Bool;
	public var _ScreenDiagonal:Float;
	public var _DistanceX:Float;
	public var _DistanceY:Float;
	public var _Distance:Float;
	public var _Direction:Float;
	public var _MinimumEasingSpeed:Float;
	public var _Margin:Float;
	public var _FastChangeSpeed:Float;
	
	/* ========================= Custom Event ========================= */
	public function _customEvent_FastSpeedChange():Void
	{
		_ChangeNumbCheck = 0;
		_FastSpeedChanging = true;
	}
	
	
	public function new(dummy:Int, actor:Actor, dummy2:Engine)
	{
		super(actor);
		nameMap.set("Actor", "actor");
		nameMap.set("OriginalSpeed", "_OriginalSpeed");
		_OriginalSpeed = 0.0;
		nameMap.set("Speed", "_Speed");
		_Speed = 30.0;
		nameMap.set("ChangeNumbCheck", "_ChangeNumbCheck");
		_ChangeNumbCheck = 0.0;
		nameMap.set("FastSpeedChanging?", "_FastSpeedChanging");
		_FastSpeedChanging = false;
		nameMap.set("Screen Diagonal", "_ScreenDiagonal");
		_ScreenDiagonal = 0.0;
		nameMap.set("Distance X", "_DistanceX");
		_DistanceX = 0.0;
		nameMap.set("Distance Y", "_DistanceY");
		_DistanceY = 0.0;
		nameMap.set("Distance", "_Distance");
		_Distance = 0.0;
		nameMap.set("Direction", "_Direction");
		_Direction = 0.0;
		nameMap.set("Minimum Easing Speed", "_MinimumEasingSpeed");
		_MinimumEasingSpeed = 5.0;
		nameMap.set("Margin", "_Margin");
		_Margin = 0.0;
		nameMap.set("FastChangeSpeed?", "_FastChangeSpeed");
		_FastChangeSpeed = 0.0;
		
	}
	
	override public function init()
	{
		
		/* ======================== When Creating ========================= */
		_OriginalSpeed = _Speed;
		_ChangeNumbCheck = 100;
		_FastSpeedChanging = false;
		_ScreenDiagonal = Math.sqrt((Math.pow(getScreenWidth(), 2) + Math.pow(getScreenHeight(), 2)));
		actor.setActorValue("ModifiedCamSpeed?", 0);
		actor.setActorValue("ShakeViewX", 0);
		actor.setActorValue("ShakeViewY", 0);
		actor.setActorValue("DragDistanceX", 0);
		actor.setActorValue("DragDistanceY", 0);
		/* "" */ runPeriodically(1000 * 0.05, function(timeTask:TimedTask):Void
		{
			_ChangeNumbCheck = (_ChangeNumbCheck + 1);
			if((_ChangeNumbCheck >= 15))
			{
				_FastSpeedChanging = false;
			}
			else
			{
				_FastSpeedChanging = true;
			}
		}, actor);
		
		/* ======================== When Updating ========================= */
		addWhenUpdatedListener(null, function(elapsedTime:Float, list:Array<Dynamic>):Void
		{
			if(wrapper.enabled)
			{
				for(actorOfType in getActorsOfType(getActorType(26)))
				{
					if(actorOfType != null && !actorOfType.dead && !actorOfType.recycled){
						_DistanceX = (actorOfType.getXCenter() - actor.getXCenter());
						_DistanceY = (actorOfType.getYCenter() - actor.getYCenter());
						_Distance = asNumber(Math.sqrt((Math.pow(_DistanceX, 2) + Math.pow(_DistanceY, 2))));
						_Direction = Utils.DEG * (Math.atan2(_DistanceY, _DistanceX));
						if((_Distance >= 0.5))
						{
							if(!(_FastSpeedChanging))
							{
								if(!((actor.getActorValue("ModifiedCam?") : Bool)))
								{
									actor.setVelocity(_Direction, (_MinimumEasingSpeed + (_Speed * ((_Distance - _Margin) / _ScreenDiagonal))));
								}
								else
								{
									actor.setVelocity(_Direction, (_MinimumEasingSpeed + (asNumber(actor.getActorValue("ModifiedCamSpeed?")) * ((_Distance - _Margin) / _ScreenDiagonal))));
								}
							}
							else
							{
								actor.setVelocity(_Direction, (_MinimumEasingSpeed + (_FastChangeSpeed * ((_Distance - _Margin) / _ScreenDiagonal))));
							}
						}
						else
						{
							actor.setVelocity(0, 0);
						}
					}
				}
				engine.moveCamera((actor.getXCenter() + asNumber(actor.getActorValue("ShakeViewX"))), (actor.getYCenter() + asNumber(actor.getActorValue("ShakeViewY"))));
			}
		});
		
	}
	
	override public function forwardMessage(msg:String)
	{
		
	}
}